package com.isu.gaswellwatch.vo.command.scss;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import com.isu.gaswellwatch.vo.command.Timing;
import jakarta.validation.constraints.NotBlank;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * 时间模式
 *
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2024/12/10 16:57
 */
@Getter
@Setter
@SuperBuilder
public class TimeMode extends Command implements Timing {

    @Serial
    private static final long serialVersionUID = -8623186068280286623L;

    /**
     * 开井时间，以秒为单位的时间
     * 150  2
     */
    @NotBlank(message = "开井时间不能为空，格式：HH:mm:ss")
    private String wellOpenTimeTimestamp;

    /**
     * 关井时间
     * 158  2
     */
    @NotBlank(message = "关井时间不能为空，格式：HH:mm:ss")
    private String wellCloseTimeTimestamp;

    /**
     * 柱塞延迟时间
     * 180  2
     */
    @NotBlank(message = "柱塞延迟时间不能为空，格式：HH:mm:ss")
    private String plungerDelayDuration;

    public TimeMode() {
        this.setCode("SCSS.TIME_MODE");
    }

    @Override
    public boolean validate() {
        return super.validate()
                && StringUtils.isNotBlank(this.wellOpenTimeTimestamp)
                && StringUtils.isNotBlank(this.wellCloseTimeTimestamp)
                && StringUtils.isNotBlank(this.plungerDelayDuration);
    }

    @Override
    protected Collection<ModbusCommandDto> builderModbusCommand() {
        List<ModbusCommandDto> resultList = new ArrayList<ModbusCommandDto>(3);
        // 地址码  功能码  起始地址  连续长度  连续字长    控制模式
        // 01     10     0032    0002     04        1(00000001, 1：时间(开)-时间(开)模式)
        resultList.add(ModbusCommandDto.builder().command("0110003200020400000001").length(16).build());

        StringBuilder command = new StringBuilder(250);
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0096    0002     04
        command.append("01100096000204");
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.wellOpenTimeTimestamp)), 8, "0"));
        resultList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        command.setLength(0);
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     009E    0002     04
        command.append("0110009E000204");
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.wellCloseTimeTimestamp)), 8, "0"));
        resultList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        command.setLength(0);
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     00B4    0002     04
        command.append("011000B4000204");
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.plungerDelayDuration)), 8, "0"));
        resultList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        return resultList;
    }

    public void setWellOpenTimeTimestamp(String wellOpenTimeTimestamp) {
        this.wellOpenTimeTimestamp = this.timingValidate(wellOpenTimeTimestamp, "wellOpenTimeTimestamp");
    }

    public void setWellCloseTimeTimestamp(String wellCloseTimeTimestamp) {
        this.wellCloseTimeTimestamp = this.timingValidate(wellCloseTimeTimestamp, "wellCloseTimeTimestamp");
    }

    public void setPlungerDelayDuration(String plungerDelayDuration) {
        this.plungerDelayDuration = this.timingValidate(plungerDelayDuration, "plungerDelayDuration");
    }

}
